"""
Early draft of a very handy utility that takes a CCDL file (Concise Campaign Definition Language)
and creates a graph(viz) visualization of it.
"""
import sys
import graphviz

name = "tb_camp.sv"
ccdl = open( sys.argv[1] ).readlines()
dot = graphviz.Digraph(name, comment='TB Cascade')

producers = {}
consumers = {}
broadcasters = ["BroadcastEvent", "SimpleHealthSeekingBehavior", "DiagnosticTreatNeg" ]

event_names = {}
event_num = 0
for camp_event in ccdl:
    pieces = camp_event.strip().split( "::" )
    if len(pieces)<2:
        continue
    #dot.node( f"{event_num}.{pieces[0]}" )
    #print( f"Processing line {camp_event}." )
    event_names[event_num] = f"[{event_num}]{pieces[0]}*{pieces[1]}*{pieces[2]}"
    triggered = pieces[3].split( "->" )
    if len(triggered)>1:
        triggers = triggered[0].strip()
        for trigger in triggers.split( "+" ):
            if trigger not in consumers:
                consumers[trigger] = set()
            for broadcaster in broadcasters:
                sender = triggered[1].split( "+" )[-1]
                if broadcaster in sender:
                    from parse import parse
                    #regex = "{}" + broadcaster + "({}){}"
                    regex = broadcaster + "({})"
                    #print( f"Looking for {regex} in {sender}" )
                    #tokens = parse( regex, triggered[1] )
                    tokens = parse( regex, sender )
                    signals = tokens[0]
                    for signal in signals.split( "/" ):
                        #print( f"Found an event broadcasting {signal}" )
                        if signal not in producers:
                            producers[signal] = set()
                        producers[signal].add( event_num )
                    
            consumers[trigger].add( event_num )
    event_num += 1

print( "PRODUCERS\n" )
print( producers )
print( "\nCONSUMERS\n" )
print( consumers )
event_whitelist = []
if len(sys.argv)>2:
    event_whitelist.append( sys.argv[2] )

for event in producers:
    if event_whitelist and event not in event_whitelist:
        continue
    senders = producers[event]
    if event in consumers:
        receivers = consumers[event]
        for from_event in senders:
            for to_event in receivers:
                ip_from = ccdl[from_event].split( "::" )[2].split("/")[1] if "/" in ccdl[from_event].split( "::" )[2] else "*"
                ip_to = ccdl[to_event].split( "::" )[2].split("/")[1] if "/" in ccdl[to_event].split( "::" )[2] else "*"
                #if ip_to != "*" and ip_from != "*" and ip_from.split(":")[0] == ip_to.split(":")[0] and ip_from.split(":")[1] != ip_to.split(":")[1]:
                if ip_to != "*" and ip_from != "*":
                    ip_from_key = ip_from.split("=")[0]
                    ip_from_value = ip_from.split("=")[1]
                    ip_to_key = ip_to.split("=")[0]
                    ip_to_value = ip_to.split("=")[1]
                    if ip_from_key == ip_to_key and ip_from_value != ip_to_value:
                        continue # aka do not continue
                time_from = ccdl[from_event].split( "::" )[0]
                time_to = ccdl[to_event].split( "::" )[0]
                time_from_start = time_from
                time_to_start = time_to
                time_from_end = 1e9
                time_to_end = 1e9
                if "-" in time_from:
                    time_from_start = float(time_from.split("-")[0])
                    time_from_end = float(time_from.split("-")[1])
                if "-" in time_to:
                    time_to_start = float(time_to.split("-")[0])
                    time_to_end = float(time_to.split("-")[1])
                time_from_start = float(time_from_start)
                time_to_start = float(time_to_start)
                if time_from_end <= time_to_start or time_from_start >= time_to_end:
                    print( f"Discarding edge from {from_event} to {to_event} because not overlapping in time." )
                    print( time_from_start, time_from_end, time_to_start, time_to_end )
                    continue
                #print( f"Creating edge from {from_event} to {to_event} for {event}." )
                print( f"{from_event} ---{event} ---> {to_event}" )
                #dot.edge( from_event, to_event, name=event )
                dot.edge( str(event_names[from_event]), str(event_names[to_event]), label=event )
        
dot.render()
display = True
if display:
    from graphviz import Source
    path = name + ".gv"
    s = Source.from_file(path)
    s.view()

